import * as i3 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { Component, ChangeDetectionStrategy } from '@angular/core';
import * as i2 from '@angular/router';
import { SchedulingRuleDtoPatternEnum } from '@smallstack/axios-api-client';
import { I18nComponent } from '@smallstack/i18n-components';
import * as i1 from '@smallstack/typesystem-client';
import { LoaderComponent } from '@smallstack/store-components';
import { IconComponent } from '@smallstack/theme-components';
import { TodoTemplateListEntryComponent } from '@smallstack/todo-components';
import { TYPE_TODO_TEMPLATES } from '@smallstack/typesystem';
import { replaceVariables } from '@smallstack/utils';
import { BaseWidgetComponent } from '@smallstack/widget-core';
import { combineLatest, switchMap } from 'rxjs';
import { C as CONFIG_LINKED_ID_FILTER, a as CONFIG_BILLING_PARTY_ID_FILTER } from './smallstack-todo-widgets-smallstack-todo-widgets-e56e58d9.mjs';

class AnytimeTodosWidgetComponent extends BaseWidgetComponent {
    constructor(rxCollectionService, activatedRoute) {
        super();
        this.rxCollectionService = rxCollectionService;
        this.activatedRoute = activatedRoute;
        this.anytimeTodos$ = combineLatest([
            this.rxCollectionService.getCollection$(TYPE_TODO_TEMPLATES),
            this.data$,
            this.context$
        ]).pipe(switchMap(([collection, data, context]) => {
            const selector = { "schedulingRule.pattern": SchedulingRuleDtoPatternEnum.Always, $or: [] };
            if (data[CONFIG_LINKED_ID_FILTER]) {
                const linkedId = replaceVariables(data[CONFIG_LINKED_ID_FILTER], {
                    ...context,
                    queryParams: this.activatedRoute.snapshot.queryParams
                });
                selector.$or.push({ "links.id": linkedId });
            }
            if (data[CONFIG_BILLING_PARTY_ID_FILTER]) {
                const billingPartyId = replaceVariables(data[CONFIG_BILLING_PARTY_ID_FILTER], {
                    ...context,
                    queryParams: this.activatedRoute.snapshot.queryParams
                });
                selector.$or.push({
                    "billingPositions.billingParties.linkedId": billingPartyId
                });
            }
            return collection.rxCollection.find({
                selector
            }).$;
        }));
    }
    newTodoCreated() {
        // this.reload();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.1.7", ngImport: i0, type: AnytimeTodosWidgetComponent, deps: [{ token: i1.RxCollectionService }, { token: i2.ActivatedRoute }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.1.7", type: AnytimeTodosWidgetComponent, isStandalone: true, selector: "smallstack-anytime-todos-widget", usesInheritance: true, ngImport: i0, template: "<ng-container *ngIf=\"anytimeTodos$ | async as anytimeTodos; else loading\">\n  <div class=\"flex flex-col\">\n    <div *ngIf=\"!anytimeTodos || anytimeTodos.length === 0\" class=\"italic text-lg text-gray-400 text-center p-5\"\n      >Keine immer verf\u00FCgbaren Aufgaben geplant</div\n    >\n    <div *ngFor=\"let anytimeTodo of anytimeTodos; ngForTrackByJson\" class=\"mb-4\">\n      <div class=\"card card-bordered card-compact shadow mb-4 relative border-l-8 bg-white text-primary\">\n        <div class=\"card-body\">\n          <div class=\"card-title flex flex-row gap-1\">\n            <smallstack-icon name=\"task\" size=\"36\" theme=\"fluency-systems-regular\"></smallstack-icon>\n            <smallstack-i18n [data]=\"anytimeTodo.label\"></smallstack-i18n>\n          </div>\n          <smallstack-i18n *ngIf=\"anytimeTodo.description\" [data]=\"anytimeTodo.description\"></smallstack-i18n>\n          <smallstack-todo-template-list-entry\n            [todoTemplate]=\"anytimeTodo\"\n            (createdTodo)=\"newTodoCreated()\"\n          ></smallstack-todo-template-list-entry>\n        </div>\n      </div>\n    </div>\n  </div>\n</ng-container>\n<ng-template #loading>\n  <smallstack-loader></smallstack-loader>\n</ng-template>\n", dependencies: [{ kind: "ngmodule", type: CommonModule }, { kind: "directive", type: i3.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "pipe", type: i3.AsyncPipe, name: "async" }, { kind: "component", type: LoaderComponent, selector: "smallstack-loader", inputs: ["loading", "color", "size", "loadingText"] }, { kind: "component", type: TodoTemplateListEntryComponent, selector: "smallstack-todo-template-list-entry", inputs: ["todoTemplate", "fixedTime", "showPopoutLink"], outputs: ["createdTodo"] }, { kind: "component", type: I18nComponent, selector: "smallstack-i18n", inputs: ["data", "showMissingKey", "replacers", "locale", "translationStore"] }, { kind: "component", type: IconComponent, selector: "smallstack-icon", inputs: ["theme", "size", "name", "alias", "action", "actionSize", "spinning", "actionSpinning", "color", "themeColor", "actionColor", "badge", "dynamic", "badgeColor"], outputs: ["iconError"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.1.7", ngImport: i0, type: AnytimeTodosWidgetComponent, decorators: [{
            type: Component,
            args: [{ selector: "smallstack-anytime-todos-widget", standalone: true, imports: [CommonModule, LoaderComponent, TodoTemplateListEntryComponent, I18nComponent, IconComponent], changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-container *ngIf=\"anytimeTodos$ | async as anytimeTodos; else loading\">\n  <div class=\"flex flex-col\">\n    <div *ngIf=\"!anytimeTodos || anytimeTodos.length === 0\" class=\"italic text-lg text-gray-400 text-center p-5\"\n      >Keine immer verf\u00FCgbaren Aufgaben geplant</div\n    >\n    <div *ngFor=\"let anytimeTodo of anytimeTodos; ngForTrackByJson\" class=\"mb-4\">\n      <div class=\"card card-bordered card-compact shadow mb-4 relative border-l-8 bg-white text-primary\">\n        <div class=\"card-body\">\n          <div class=\"card-title flex flex-row gap-1\">\n            <smallstack-icon name=\"task\" size=\"36\" theme=\"fluency-systems-regular\"></smallstack-icon>\n            <smallstack-i18n [data]=\"anytimeTodo.label\"></smallstack-i18n>\n          </div>\n          <smallstack-i18n *ngIf=\"anytimeTodo.description\" [data]=\"anytimeTodo.description\"></smallstack-i18n>\n          <smallstack-todo-template-list-entry\n            [todoTemplate]=\"anytimeTodo\"\n            (createdTodo)=\"newTodoCreated()\"\n          ></smallstack-todo-template-list-entry>\n        </div>\n      </div>\n    </div>\n  </div>\n</ng-container>\n<ng-template #loading>\n  <smallstack-loader></smallstack-loader>\n</ng-template>\n" }]
        }], ctorParameters: function () { return [{ type: i1.RxCollectionService }, { type: i2.ActivatedRoute }]; } });

export { AnytimeTodosWidgetComponent };
//# sourceMappingURL=smallstack-todo-widgets-anytime-todos-widget.component-091dd757.mjs.map
