import * as i3 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { Component, ChangeDetectionStrategy } from '@angular/core';
import * as i2 from '@angular/router';
import { SchedulingRuleDtoPatternEnum } from '@smallstack/axios-api-client';
import * as i4 from '@smallstack/common-components';
import * as i1 from '@smallstack/i18n-components';
import { I18nComponent, DateComponent } from '@smallstack/i18n-components';
import * as i1$1 from '@smallstack/typesystem-client';
import { TypeModelComponent } from '@smallstack/typesystem-client';
import { LoaderComponent } from '@smallstack/store-components';
import { IconComponent } from '@smallstack/theme-components';
import * as i1$2 from '@smallstack/todo-components';
import { generateNextDatesUntil, TodoListEntryComponent, TodoTemplateListEntryComponent } from '@smallstack/todo-components';
import { TYPE_TODOS, TYPE_TODO_TEMPLATES, TYPE_REPLACEMENTS, TYPE_CUSTOMERS } from '@smallstack/typesystem';
import { replaceVariables, ONE_DAY, ONE_HOUR, asyncFilter, getGoogleMapsNavigationUrl } from '@smallstack/utils';
import { BaseWidgetComponent } from '@smallstack/widget-core';
import { subDays, startOfDay, endOfDay, format, addDays } from 'date-fns';
import { combineLatest, switchMap, of, map } from 'rxjs';
import { b as CONFIG_ONLY_MINE, c as CONFIG_LINKED_ID_FILTER } from './smallstack-todo-widgets-smallstack-todo-widgets-e56e58d9.mjs';

/* eslint-disable complexity */
/* eslint-disable max-lines-per-function */
class TodoAgendaWidgetComponent extends BaseWidgetComponent {
    constructor(notificationService, translationStore, activatedRoute, rxCollectionService, contextService, myTodoStore) {
        super();
        this.notificationService = notificationService;
        this.translationStore = translationStore;
        this.activatedRoute = activatedRoute;
        this.rxCollectionService = rxCollectionService;
        this.contextService = contextService;
        this.myTodoStore = myTodoStore;
        this.agendaDays$ = combineLatest([
            this.getContextReplacedData$(CONFIG_ONLY_MINE),
            this.context$,
            this.rxCollectionService.getCollection$(TYPE_TODOS),
            this.rxCollectionService.getCollection$(TYPE_TODO_TEMPLATES),
            this.rxCollectionService.getCollection$(TYPE_REPLACEMENTS)
        ]).pipe(switchMap(async ([onlyMine, context, todoCollection, todoTemplatesCollection, replacementsCollection]) => {
            await todoCollection.sync();
            await todoTemplatesCollection.sync();
            await replacementsCollection.sync();
            return [onlyMine, context, todoCollection, todoTemplatesCollection, replacementsCollection];
        }), switchMap(([onlyMine, context, todoCollection, todoTemplatesCollection, replacementsCollection]) => {
            return combineLatest([
                of(onlyMine),
                of(context),
                todoCollection.rxCollection.find().$,
                todoTemplatesCollection.rxCollection.find().$,
                replacementsCollection.rxCollection.find().$,
                this.data$
            ]);
        }), map(([onlyMine, context, todos, todoTemplates, replacements, data]) => {
            if (data[CONFIG_LINKED_ID_FILTER]) {
                const linkedId = replaceVariables(data[CONFIG_LINKED_ID_FILTER], {
                    ...context,
                    queryParams: this.activatedRoute.snapshot.queryParams
                });
                const filteredTodoTemplates = todoTemplates.filter((t) => {
                    if (t.billingPositions instanceof Array && t.billingPositions.length > 0)
                        for (const billingPosition of t.billingPositions) {
                            for (const billingParty of billingPosition.billingParties) {
                                if (billingParty.linkedId === linkedId)
                                    return true;
                            }
                        }
                    return false;
                });
                return [onlyMine, context, todos, filteredTodoTemplates, replacements, data];
            }
            return [onlyMine, context, todos, todoTemplates, replacements, data];
        }), switchMap(async ([onlyMine, context, todos, todoTemplates, replacements, data]) => {
            const agendaDays = [];
            const upcomingTodos = [];
            const overdueTodos = [];
            for (const todoTemplate of todoTemplates) {
                if (todoTemplate.schedulingRule?.pattern &&
                    todoTemplate.schedulingRule?.pattern !== SchedulingRuleDtoPatternEnum.Always) {
                    const nextDates = generateNextDatesUntil(todoTemplate.schedulingRule, {
                        from: Date.now(),
                        until: Date.now() + ONE_DAY * 14
                    });
                    if (Array.isArray(nextDates))
                        for (const nextDate of nextDates) {
                            // try to find a todo
                            const todo = todos.find((t) => t.todoTemplateRefId === todoTemplate.id && t.startTime === nextDate);
                            if (!todo || todo.done !== true)
                                upcomingTodos.push({
                                    startTime: nextDate,
                                    endTime: nextDate + ONE_HOUR,
                                    todoTemplate: todoTemplate?.toMutableJSON(),
                                    todo: todo?.toMutableJSON()
                                });
                        }
                    const previousDates = generateNextDatesUntil(todoTemplate.schedulingRule, {
                        from: Date.now() - ONE_DAY * 14,
                        until: Date.now()
                    });
                    if (Array.isArray(previousDates))
                        for (const previousDate of previousDates) {
                            // try to find a todo
                            const todo = todos.find((t) => t.todoTemplateRefId === todoTemplate.id && t.startTime === previousDate);
                            if (!todo || todo.done !== true)
                                overdueTodos.push({
                                    startTime: previousDate,
                                    endTime: previousDate + ONE_HOUR,
                                    todoTemplate: todoTemplate?.toMutableJSON(),
                                    todo: todo?.toMutableJSON()
                                });
                        }
                }
            }
            // get overdue todos of past 14 days
            for (let i = 14; i > 0; i--) {
                const day = subDays(new Date(), i);
                const dayStart = startOfDay(day).valueOf();
                const dayEnd = endOfDay(day).valueOf();
                const todoDates = overdueTodos
                    .filter((ut) => ut.startTime < dayEnd && ut.startTime >= dayStart)
                    .sort((a, b) => a.startTime - b.startTime);
                if (todoDates?.length > 0) {
                    todoDates.forEach((todoDate) => (todoDate.overdue = true));
                    agendaDays.push({
                        date: format(day, "EE, dd. MMM yyyy"),
                        todoDates
                    });
                }
            }
            // compute next 14 days
            for (let i = 0; i < 14; i++) {
                const day = addDays(new Date(), i);
                const dayStart = startOfDay(day).valueOf();
                const dayEnd = endOfDay(day).valueOf();
                const todoDates = upcomingTodos
                    .filter((ut) => ut.startTime < dayEnd && ut.startTime >= dayStart)
                    .sort((a, b) => a.startTime - b.startTime);
                if (todoDates?.length > 0) {
                    agendaDays.push({
                        date: format(day, "EE, dd. MMM yyyy"),
                        todoDates
                    });
                }
            }
            // filter by ownerId ?
            if (onlyMine) {
                const currentUserId = this.contextService.getEvaluatedContext().userId;
                if (!currentUserId)
                    throw new Error("No userId found in context!");
                for (const agendaDay of agendaDays) {
                    agendaDay.todoDates = await asyncFilter(agendaDay.todoDates, async (todoDate) => {
                        // not assigned to anyone
                        if (!todoDate.todoTemplate.ownerIds?.length)
                            return true;
                        // directly assigned ones
                        if (todoDate.todoTemplate.ownerIds?.includes(currentUserId)) {
                            // get replacements if available
                            for (const replacement of replacements) {
                                if (replacement.start <= todoDate.startTime && replacement.end >= todoDate.endTime)
                                    for (const replacer of replacement.replacers) {
                                        if (replacer.targetId === todoDate.todoTemplate.id) {
                                            todoDate.statusText =
                                                "Vertreten durch " + (await this.getDisplayNameById(replacer.newValue[0]));
                                            todoDate.disabled = true;
                                        }
                                    }
                            }
                            return true;
                        }
                        // assigned via replacements
                        for (const replacement of replacements) {
                            if (replacement.start <= todoDate.startTime && replacement.end >= todoDate.endTime)
                                for (const replacer of replacement.replacers) {
                                    if (replacer.targetId === todoDate.todoTemplate.id &&
                                        replacer.newValue instanceof Array &&
                                        replacer.newValue.includes(currentUserId)) {
                                        todoDate.statusText =
                                            "Sie vertreten " + (await this.getDisplayNameById(todoDate.todoTemplate.ownerIds[0]));
                                        return true;
                                    }
                                }
                        }
                        return false;
                    });
                }
            }
            return agendaDays;
        }));
    }
    openGoogleMaps(address) {
        window.open(getGoogleMapsNavigationUrl(address), "_blank");
    }
    async createTodo(todo) {
        await this.myTodoStore.put(todo);
        if (todo.done) {
            this.notificationService.popup.success("Aufgabe erledigt", "Die Aufgabe '" +
                this.translationStore.translate(todo.label) +
                "' ist vollständig erledigt und wurde nun aus der täglichen Agenda entfernt.");
            this.refreshTodoCollections();
        }
    }
    newTodoCreated() {
        this.refreshTodoCollections();
    }
    refreshTodoCollections() {
        void this.rxCollectionService.getCollection(TYPE_TODO_TEMPLATES).then(async (collection) => {
            await collection.sync();
        });
        void this.rxCollectionService.getCollection(TYPE_TODOS).then(async (collection) => {
            await collection.sync();
        });
    }
    async getDisplayNameById(userId) {
        return this.rxCollectionService.getCollection(TYPE_CUSTOMERS).then(async (collection) => {
            await collection.replicationState.awaitInSync();
            return collection.rxCollection
                .findOne({ selector: { id: userId } })
                .exec()
                .then((user) => {
                if (user)
                    return user.displayName;
                return userId;
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.1.7", ngImport: i0, type: TodoAgendaWidgetComponent, deps: [{ token: i1.NotificationService }, { token: i1.TranslationStore }, { token: i2.ActivatedRoute }, { token: i1$1.RxCollectionService }, { token: i4.ContextService }, { token: i1$2.MyTodoStore }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.1.7", type: TodoAgendaWidgetComponent, isStandalone: true, selector: "smallstack-todo-agenda-widget", usesInheritance: true, ngImport: i0, template: "<ng-container *ngIf=\"agendaDays$ | async as agendaDays; else loading\">\n  <div class=\"flex flex-col\">\n    <div *ngIf=\"!agendaDays || agendaDays.length === 0\" class=\"italic text-lg text-gray-400 text-center p-5\"\n      >Aktuell keine Aufgaben geplant</div\n    >\n    <div *ngFor=\"let todoDay of agendaDays; ngForTrackByJson\" class=\"mb-4\">\n      <div *ngIf=\"todoDay.fadeOut !== true\" class=\"text-lg my-2 flex flex-row gap-1 justify-start items-center\">\n        <smallstack-icon name=\"calendar\" size=\"24\"></smallstack-icon>\n        <span>{{ todoDay.date }}</span>\n      </div>\n      <div>\n        <div *ngIf=\"!todoDay.todoDates || todoDay.todoDates.length === 0\">\n          <div class=\"italic text-sm text-gray-400\">Keine Aufgaben an diesem Tag</div>\n        </div>\n        <ng-container *ngFor=\"let todoDate of todoDay.todoDates; ngForTrackByJson\">\n          <div\n            *ngIf=\"todoDate.invisible !== true\"\n            class=\"card card-bordered card-compact shadow mb-4 relative\"\n            [ngClass]=\"{\n              'border-l-8': todoDate.todo,\n              'border-l-success': todoDate.todo && !todoDate.todo.done,\n              'bg-error text-error-content': todoDate.overdue && !todoDate.todo,\n              'bg-white text-primary': !todoDate.overdue,\n              'border-l-primary': todoDate.todo?.done\n            }\"\n          >\n            <div\n              *ngIf=\"todoDate.fadeOut\"\n              class=\"absolute w-full h-full z-40 flex flex-row justify-center items-center content-center\"\n            >\n              <button (click)=\"todoDate.invisible = true\" class=\"btn btn-primary\">\n                Abgeschlossene Aufgabe ausblenden\n              </button>\n            </div>\n            <div\n              *ngIf=\"todoDate.disabled\"\n              class=\"absolute w-full h-full z-50 bg-gray-500 bg-opacity-50 rounded-2xl\"\n            ></div>\n            <div\n              class=\"card-body\"\n              [ngClass]=\"{\n                'transition-opacity ease-out duration-1000 opacity-20': todoDate.fadeOut\n              }\"\n            >\n              <div class=\"card-title flex flex-row gap-1\">\n                <smallstack-icon\n                  *ngIf=\"todoDate.todo && todoDate.todo.done !== true\"\n                  name=\"play-button-circled--v1\"\n                  size=\"36\"\n                  class=\"animate-pulse\"\n                  theme=\"fluency-systems-regular\"\n                  themeColor=\"daisyTheme.success.bgColor\"\n                ></smallstack-icon>\n                <smallstack-icon\n                  *ngIf=\"todoDate.todo && todoDate.todo.done === true\"\n                  name=\"checked-checkbox\"\n                  size=\"36\"\n                  theme=\"fluency-systems-regular\"\n                  themeColor=\"daisyTheme.primary.bgColor\"\n                ></smallstack-icon>\n                <span *ngIf=\"todoDate.startTime\">\n                  {{ todoDate.startTime | date: \"HH:mm\" }}\n                </span>\n                <smallstack-i18n [data]=\"todoDate.todoTemplate.label\"></smallstack-i18n>\n              </div>\n              <div *ngIf=\"todoDate.statusText\" class=\"badge badge-info\">\n                {{ todoDate.statusText }}\n              </div>\n              <div *ngIf=\"todoDate.overdue\" class=\"font-bold\"\n                >Aufgabe \u00FCberf\u00E4llig seit <smallstack-date [timestamp]=\"todoDate.endTime\"></smallstack-date\n              ></div>\n              <smallstack-i18n\n                *ngIf=\"todoDate.todoTemplate.description\"\n                [data]=\"todoDate.todoTemplate.description\"\n              ></smallstack-i18n>\n              <ng-container *ngIf=\"todoDate.todo; else startNew\">\n                <smallstack-todo-list-entry\n                  [todo]=\"todoDate.todo\"\n                  (saveFn)=\"createTodo($event)\"\n                ></smallstack-todo-list-entry>\n              </ng-container>\n              <ng-template #startNew>\n                <smallstack-todo-template-list-entry\n                  [todoTemplate]=\"todoDate.todoTemplate\"\n                  [fixedTime]=\"todoDate.startTime\"\n                  (createdTodo)=\"newTodoCreated()\"\n                ></smallstack-todo-template-list-entry>\n                <!-- <button\n                  [loadingFn]=\"createTodo(todoDate.todoTemplate, todoDate.startTime)\"\n                  class=\"btn btn-xl btn-block btn-primary\"\n                  >Aufgabe starten</button\n                > -->\n              </ng-template>\n            </div>\n          </div>\n        </ng-container>\n      </div>\n    </div>\n  </div>\n</ng-container>\n<ng-template #loading>\n  <smallstack-loader></smallstack-loader>\n</ng-template>\n", styles: [""], dependencies: [{ kind: "ngmodule", type: CommonModule }, { kind: "directive", type: i3.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i3.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "pipe", type: i3.AsyncPipe, name: "async" }, { kind: "pipe", type: i3.DatePipe, name: "date" }, { kind: "component", type: LoaderComponent, selector: "smallstack-loader", inputs: ["loading", "color", "size", "loadingText"] }, { kind: "component", type: IconComponent, selector: "smallstack-icon", inputs: ["theme", "size", "name", "alias", "action", "actionSize", "spinning", "actionSpinning", "color", "themeColor", "actionColor", "badge", "dynamic", "badgeColor"], outputs: ["iconError"] }, { kind: "component", type: I18nComponent, selector: "smallstack-i18n", inputs: ["data", "showMissingKey", "replacers", "locale", "translationStore"] }, { kind: "component", type: DateComponent, selector: "smallstack-date", inputs: ["timestamp", "format"] }, { kind: "component", type: TodoListEntryComponent, selector: "smallstack-todo-list-entry", inputs: ["todo", "editable", "showDueDate", "showChecklistEntries", "isRoot"], outputs: ["saveFn"] }, { kind: "component", type: TodoTemplateListEntryComponent, selector: "smallstack-todo-template-list-entry", inputs: ["todoTemplate", "fixedTime", "showPopoutLink"], outputs: ["createdTodo"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.1.7", ngImport: i0, type: TodoAgendaWidgetComponent, decorators: [{
            type: Component,
            args: [{ selector: "smallstack-todo-agenda-widget", changeDetection: ChangeDetectionStrategy.OnPush, standalone: true, imports: [
                        CommonModule,
                        LoaderComponent,
                        IconComponent,
                        I18nComponent,
                        DateComponent,
                        TypeModelComponent,
                        TodoListEntryComponent,
                        TodoTemplateListEntryComponent
                    ], template: "<ng-container *ngIf=\"agendaDays$ | async as agendaDays; else loading\">\n  <div class=\"flex flex-col\">\n    <div *ngIf=\"!agendaDays || agendaDays.length === 0\" class=\"italic text-lg text-gray-400 text-center p-5\"\n      >Aktuell keine Aufgaben geplant</div\n    >\n    <div *ngFor=\"let todoDay of agendaDays; ngForTrackByJson\" class=\"mb-4\">\n      <div *ngIf=\"todoDay.fadeOut !== true\" class=\"text-lg my-2 flex flex-row gap-1 justify-start items-center\">\n        <smallstack-icon name=\"calendar\" size=\"24\"></smallstack-icon>\n        <span>{{ todoDay.date }}</span>\n      </div>\n      <div>\n        <div *ngIf=\"!todoDay.todoDates || todoDay.todoDates.length === 0\">\n          <div class=\"italic text-sm text-gray-400\">Keine Aufgaben an diesem Tag</div>\n        </div>\n        <ng-container *ngFor=\"let todoDate of todoDay.todoDates; ngForTrackByJson\">\n          <div\n            *ngIf=\"todoDate.invisible !== true\"\n            class=\"card card-bordered card-compact shadow mb-4 relative\"\n            [ngClass]=\"{\n              'border-l-8': todoDate.todo,\n              'border-l-success': todoDate.todo && !todoDate.todo.done,\n              'bg-error text-error-content': todoDate.overdue && !todoDate.todo,\n              'bg-white text-primary': !todoDate.overdue,\n              'border-l-primary': todoDate.todo?.done\n            }\"\n          >\n            <div\n              *ngIf=\"todoDate.fadeOut\"\n              class=\"absolute w-full h-full z-40 flex flex-row justify-center items-center content-center\"\n            >\n              <button (click)=\"todoDate.invisible = true\" class=\"btn btn-primary\">\n                Abgeschlossene Aufgabe ausblenden\n              </button>\n            </div>\n            <div\n              *ngIf=\"todoDate.disabled\"\n              class=\"absolute w-full h-full z-50 bg-gray-500 bg-opacity-50 rounded-2xl\"\n            ></div>\n            <div\n              class=\"card-body\"\n              [ngClass]=\"{\n                'transition-opacity ease-out duration-1000 opacity-20': todoDate.fadeOut\n              }\"\n            >\n              <div class=\"card-title flex flex-row gap-1\">\n                <smallstack-icon\n                  *ngIf=\"todoDate.todo && todoDate.todo.done !== true\"\n                  name=\"play-button-circled--v1\"\n                  size=\"36\"\n                  class=\"animate-pulse\"\n                  theme=\"fluency-systems-regular\"\n                  themeColor=\"daisyTheme.success.bgColor\"\n                ></smallstack-icon>\n                <smallstack-icon\n                  *ngIf=\"todoDate.todo && todoDate.todo.done === true\"\n                  name=\"checked-checkbox\"\n                  size=\"36\"\n                  theme=\"fluency-systems-regular\"\n                  themeColor=\"daisyTheme.primary.bgColor\"\n                ></smallstack-icon>\n                <span *ngIf=\"todoDate.startTime\">\n                  {{ todoDate.startTime | date: \"HH:mm\" }}\n                </span>\n                <smallstack-i18n [data]=\"todoDate.todoTemplate.label\"></smallstack-i18n>\n              </div>\n              <div *ngIf=\"todoDate.statusText\" class=\"badge badge-info\">\n                {{ todoDate.statusText }}\n              </div>\n              <div *ngIf=\"todoDate.overdue\" class=\"font-bold\"\n                >Aufgabe \u00FCberf\u00E4llig seit <smallstack-date [timestamp]=\"todoDate.endTime\"></smallstack-date\n              ></div>\n              <smallstack-i18n\n                *ngIf=\"todoDate.todoTemplate.description\"\n                [data]=\"todoDate.todoTemplate.description\"\n              ></smallstack-i18n>\n              <ng-container *ngIf=\"todoDate.todo; else startNew\">\n                <smallstack-todo-list-entry\n                  [todo]=\"todoDate.todo\"\n                  (saveFn)=\"createTodo($event)\"\n                ></smallstack-todo-list-entry>\n              </ng-container>\n              <ng-template #startNew>\n                <smallstack-todo-template-list-entry\n                  [todoTemplate]=\"todoDate.todoTemplate\"\n                  [fixedTime]=\"todoDate.startTime\"\n                  (createdTodo)=\"newTodoCreated()\"\n                ></smallstack-todo-template-list-entry>\n                <!-- <button\n                  [loadingFn]=\"createTodo(todoDate.todoTemplate, todoDate.startTime)\"\n                  class=\"btn btn-xl btn-block btn-primary\"\n                  >Aufgabe starten</button\n                > -->\n              </ng-template>\n            </div>\n          </div>\n        </ng-container>\n      </div>\n    </div>\n  </div>\n</ng-container>\n<ng-template #loading>\n  <smallstack-loader></smallstack-loader>\n</ng-template>\n" }]
        }], ctorParameters: function () { return [{ type: i1.NotificationService }, { type: i1.TranslationStore }, { type: i2.ActivatedRoute }, { type: i1$1.RxCollectionService }, { type: i4.ContextService }, { type: i1$2.MyTodoStore }]; } });

export { TodoAgendaWidgetComponent };
//# sourceMappingURL=smallstack-todo-widgets-todo-agenda-widget.component-62a4ffd9.mjs.map
