import * as i3 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { Component, ChangeDetectionStrategy, ViewChild } from '@angular/core';
import * as i4 from '@angular/forms';
import { FormsModule } from '@angular/forms';
import * as i5 from '@angular/material/legacy-table';
import { MatLegacyTableModule } from '@angular/material/legacy-table';
import * as i2 from '@smallstack/axios-api-client';
import { ReplacementsApi } from '@smallstack/axios-api-client';
import { LoadingElementDirective, LetDirective } from '@smallstack/common-components';
import { WIDGET_FORM_INPUT_I18N } from '@smallstack/core-common';
import { FORM_INPUT_DATE_TIME } from '@smallstack/form-shared';
import { I18nComponent, DateComponent } from '@smallstack/i18n-components';
import { RxModelComponent, RxPropertyComponent, RxSelectComponent } from '@smallstack/rx-components';
import { MatSortStoreDirective, StorePropertyComponent } from '@smallstack/store-components';
import { IconComponent } from '@smallstack/theme-components';
import { ScheduledTimesComponent } from '@smallstack/todo-components';
import { TYPE_CUSTOMERS, TYPE_TODO_TEMPLATES, CollectionNames } from '@smallstack/typesystem';
import * as i1 from '@smallstack/typesystem-client';
import { filterNullish, isNonEmptyString, cloneObject } from '@smallstack/utils';
import * as i6 from '@smallstack/widget-core';
import { BaseWidgetComponent, FormComponent, SmallstackFormCoreModule } from '@smallstack/widget-core';
import { BehaviorSubject, combineLatest, switchMap, take, firstValueFrom } from 'rxjs';

class TodoTemplateOwnerReplacementEditorWidgetComponent extends BaseWidgetComponent {
    set replacement(replacement) {
        this.replacement$.next(replacement);
    }
    get replacement() {
        return this.replacement$.getValue();
    }
    constructor(rxCollectionService, axiosApiClient) {
        super();
        this.rxCollectionService = rxCollectionService;
        this.axiosApiClient = axiosApiClient;
        this.replacement$ = new BehaviorSubject(undefined);
        this.currentOwnerFilter$ = new BehaviorSubject(undefined);
        this.currentSearch$ = new BehaviorSubject(undefined);
        this.customers$ = combineLatest([
            this.rxCollectionService.getCollection$(TYPE_CUSTOMERS),
            this.currentOwnerFilter$
        ]).pipe(switchMap(([customerCollection, currentOwnerFilter]) => {
            if (currentOwnerFilter)
                return customerCollection.rxCollection.find({ selector: { id: { $ne: currentOwnerFilter } } }).$;
            return customerCollection.rxCollection.find().$;
        }));
        this.currentTodoTemplates$ = combineLatest([
            this.rxCollectionService.getCollection$(TYPE_TODO_TEMPLATES),
            this.replacement$,
            this.currentSearch$
        ]).pipe(filterNullish({ filterEmptyObjects: true }), switchMap(([collection, replacement, currentSearch]) => {
            if (!currentSearch)
                return undefined;
            // create search
            return collection.rxCollection.find({ selector: currentSearch }).$;
            //.pipe(map((todoTemplates) => [todoTemplates, replacement]));
            // if (
            //   replacement.customerId === undefined ||
            //   !replacement.end ||
            //   !replacement.start
            // )
            //   return of([undefined, undefined]);
        })
        // filterNullish({ filterEmptyObjects: true }),
        // map(([todoTemplates, replacement]: [TodoTemplateDto[], ReplacementDto]) => {
        //   // generate todos for the given time range
        //   const entries: ReplacementTableEntry[] = [];
        //   for (const todoTemplate of todoTemplates) {
        //     entries.push({
        //       id: todoTemplate.id,
        //       label: todoTemplate.label,
        //       description: todoTemplate.description,
        //       billingParties: todoTemplate.billingPositions.map((billingPosition) => billingPosition.billingParties).flat()
        //     });
        //   }
        //   return entries;
        // })
        );
        this.replacersTableColumns = ["facility", "label", "description", "date", "replacers", "actions"];
        this.newReplacersTableColumns = [
            "checkboxes",
            "ownerIds",
            "facility",
            "label",
            "description",
            "date",
            "actions"
        ];
        this.formSchema = {
            type: "object",
            properties: {
                label: {
                    title: "Bezeichnung",
                    type: "array",
                    "x-schema-form": {
                        inputWidget: WIDGET_FORM_INPUT_I18N
                    }
                },
                start: {
                    title: "Von",
                    type: "number",
                    "x-schema-form": {
                        widget: FORM_INPUT_DATE_TIME
                    }
                },
                end: {
                    title: "Bis",
                    type: "number",
                    "x-schema-form": {
                        widget: FORM_INPUT_DATE_TIME
                    }
                }
            },
            required: ["label", "start", "end"]
        };
        this.checkedTodoTemplates = [];
        this.allChecked = false;
        this.subscription.add(this.context$.subscribe((context) => {
            if (context?.id) {
                const replacement = context;
                this.replacement = replacement;
            }
        }));
    }
    handleSocketData(socketName, data) {
        switch (socketName) {
            case "save":
                return this.createReplacements();
            case "validate":
                return this.validate();
        }
    }
    async validate() {
        await this.formComponent.validateForm()();
        // if (this.checkedTodoTemplates.length === 0) return false;
        const replacement = this.replacement$.getValue();
        if (
        // replacement?.customerId === undefined ||
        !replacement?.end ||
            !replacement?.start)
            return false;
        return true;
    }
    async createReplacements() {
        if (await this.validate()) {
            let replacement = this.replacement$.getValue();
            if (replacement.id)
                replacement = (await this.axiosApiClient.get(ReplacementsApi).updateReplacement({ id: replacement.id, replacement })).data;
            else
                replacement = (await this.axiosApiClient.get(ReplacementsApi).createReplacement({ replacement })).data;
            await this.rxCollectionService.sync(CollectionNames.COLLECTION_REPLACEMENTS);
            return replacement;
        }
        return undefined;
    }
    toggleAll(event) {
        if (event.target.checked)
            void this.currentTodoTemplates$.pipe(take(1)).subscribe((todoTemplates) => {
                this.checkedTodoTemplates = todoTemplates;
            });
        else
            this.checkedTodoTemplates = [];
        this.allChecked = event.target.checked;
    }
    async toggle(element) {
        if (this.isChecked(element.id))
            this.checkedTodoTemplates = this.checkedTodoTemplates.filter((e) => e.id !== element.id);
        else
            this.checkedTodoTemplates.push(element);
        this.allChecked = this.checkedTodoTemplates.length === (await firstValueFrom(this.currentTodoTemplates$)).length;
    }
    isChecked(id) {
        return this.checkedTodoTemplates.some((todoTemplate) => todoTemplate.id === id);
    }
    isAlreadyReplaced(todoTemplateId) {
        return this.replacement?.replacers?.some((replacer) => replacer.targetId === todoTemplateId);
    }
    addSearchFilter(filterName, filterValue) {
        // set ownerId subject
        if (filterName === "ownerIds")
            this.currentOwnerFilter$.next(filterValue);
        // add search filter
        let currentSearch = this.currentSearch$.getValue();
        if (!currentSearch)
            currentSearch = {};
        if (isNonEmptyString(filterValue))
            currentSearch[filterName] = filterValue;
        else
            delete currentSearch[filterName];
        this.currentSearch$.next(currentSearch);
        this.checkedTodoTemplates = [];
        this.allChecked = false;
        this.cdr.markForCheck();
    }
    addReplacer(replacementId, todoTemplateId) {
        const currentReplacement = cloneObject(this.replacement);
        const currentReplacer = currentReplacement.replacers?.find((replacer) => replacer.targetId === todoTemplateId);
        if (currentReplacer)
            currentReplacer.newValue = [replacementId];
        else {
            if (!currentReplacement.replacers)
                currentReplacement.replacers = [];
            currentReplacement.replacers.push({
                targetId: todoTemplateId,
                targetType: TYPE_TODO_TEMPLATES,
                propertyName: "ownerIds",
                newValue: [replacementId]
            });
        }
        this.replacement = currentReplacement;
        this.cdr.markForCheck();
    }
    addManyReplacers() {
        const currentReplacement = cloneObject(this.replacement);
        for (const checkedTodoTemplate of this.checkedTodoTemplates) {
            const currentReplacer = currentReplacement.replacers?.find((replacer) => replacer.targetId === checkedTodoTemplate.id);
            if (currentReplacer)
                currentReplacer.newValue = [this.manyReplacerId];
            else {
                if (!currentReplacement.replacers)
                    currentReplacement.replacers = [];
                currentReplacement.replacers.push({
                    targetId: checkedTodoTemplate.id,
                    targetType: TYPE_TODO_TEMPLATES,
                    propertyName: "ownerIds",
                    newValue: [this.manyReplacerId]
                });
            }
        }
        this.allChecked = false;
        this.manyReplacerId = undefined;
        this.checkedTodoTemplates = [];
        this.replacement = currentReplacement;
        this.cdr.markForCheck();
    }
    removeReplacer(targetId) {
        const currentReplacement = this.replacement;
        currentReplacement.replacers = currentReplacement.replacers.filter((replacer) => replacer.targetId !== targetId);
        this.replacement = currentReplacement;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.1.7", ngImport: i0, type: TodoTemplateOwnerReplacementEditorWidgetComponent, deps: [{ token: i1.RxCollectionService }, { token: i2.AxiosApiClient }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.1.7", type: TodoTemplateOwnerReplacementEditorWidgetComponent, isStandalone: true, selector: "smallstack-todo-template-owner-replacement-widget", viewQueries: [{ propertyName: "formComponent", first: true, predicate: FormComponent, descendants: true }], usesInheritance: true, ngImport: i0, template: "<div class=\"p-2\">\n  <div class=\"text-lg font-bold\">Zeitraum</div>\n  <smallstack-form [schema]=\"formSchema\" (valueChange)=\"replacement$.next($event)\" [value]=\"replacement$ | async\">\n    <smallstack-form-table></smallstack-form-table>\n  </smallstack-form>\n  <div class=\"divider\"></div>\n  <div class=\"text-lg font-bold\">Aufgaben Vertretungen</div>\n  <table mat-table [dataSource]=\"(replacement$ | async)?.replacers\">\n    <ng-container matColumnDef=\"facility\">\n      <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Geb\u00E4ude / Anlage </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <smallstack-rx-model [type]=\"element.targetType\" [modelId]=\"element.targetId\">\n          <ng-template let-model=\"model\">\n            <ng-container *ngFor=\"let billingPosition of model?.billingPositions\">\n              <smallstack-rx-property\n                *ngFor=\"let billingParty of billingPosition.billingParties\"\n                [type]=\"billingParty.linkedType\"\n                [modelId]=\"billingParty.linkedId\"\n                property=\"label\"\n              ></smallstack-rx-property>\n            </ng-container>\n          </ng-template>\n        </smallstack-rx-model>\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"label\">\n      <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Aufgabe </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <smallstack-rx-model [type]=\"element.targetType\" [modelId]=\"element.targetId\">\n          <ng-template let-model=\"model\">\n            <smallstack-i18n [data]=\"model?.label\"></smallstack-i18n>\n          </ng-template>\n        </smallstack-rx-model>\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"description\">\n      <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Beschreibung </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <smallstack-rx-model [type]=\"element.targetType\" [modelId]=\"element.targetId\">\n          <ng-template let-model=\"model\">\n            <smallstack-i18n [data]=\"model?.description\"></smallstack-i18n>\n          </ng-template>\n        </smallstack-rx-model>\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"date\">\n      <th mat-header-cell *matHeaderCellDef> Geplante Zeiten </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <smallstack-rx-model [type]=\"element.targetType\" [modelId]=\"element.targetId\">\n          <ng-template let-model=\"model\">\n            <smallstack-scheduled-times\n              [start]=\"(replacement$ | async)?.start\"\n              [end]=\"(replacement$ | async)?.end\"\n              [schedulingRule]=\"model?.schedulingRule\"\n            ></smallstack-scheduled-times>\n          </ng-template>\n        </smallstack-rx-model>\n        <!-- <div class=\"flex flex-row flex-wrap gap-1\">\n          <div *ngFor=\"let date of element.dates\" class=\"badge badge-primary\">\n            <smallstack-date [timestamp]=\"date\"></smallstack-date>\n          </div>\n        </div> -->\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"replacers\">\n      <th mat-header-cell *matHeaderCellDef> Vertretung </th>\n      <td mat-cell *matCellDef=\"let element\" class=\"whitespace-nowrap\">\n        <ng-container *ngFor=\"let customer of element.newValue\">\n          <smallstack-rx-property type=\"customers\" [modelId]=\"customer\" property=\"displayName\">\n          </smallstack-rx-property>\n        </ng-container>\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"actions\">\n      <th mat-header-cell *matHeaderCellDef> </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <button class=\"btn-action btn-primary\" (click)=\"removeReplacer(element.targetId)\">\n          <smallstack-icon name=\"trash\"></smallstack-icon>\n        </button>\n      </td>\n    </ng-container>\n    <tr mat-header-row *matHeaderRowDef=\"replacersTableColumns\"></tr>\n    <tr mat-row *matRowDef=\"let row; columns: replacersTableColumns\"></tr>\n  </table>\n  <div\n    *ngIf=\"(replacement$ | async)?.replacers === undefined || (replacement$ | async)?.replacers?.length === 0\"\n    class=\"text-center text-help m-6\"\n  >\n    Keine Vertretungen vorhanden\n  </div>\n  <div class=\"divider\"></div>\n  <div class=\"text-lg font-bold\">Neue Vertretungen hinzuf\u00FCgen</div>\n  <div class=\"flex flex-row gap-2 py-3 outline-border\">\n    <div class=\"title\">Filter</div>\n    <!-- customer filter -->\n    <smallstack-rx-select\n      label=\"Hausmeister\"\n      type=\"customers\"\n      displayProperty=\"displayName\"\n      valueProperty=\"id\"\n      (valueChange)=\"addSearchFilter('ownerIds', $event)\"\n    ></smallstack-rx-select>\n\n    <!-- facility filter -->\n    <smallstack-rx-select\n      label=\"Geb\u00E4ude / Anlage\"\n      type=\"custom_facilities\"\n      displayProperty=\"label\"\n      valueProperty=\"id\"\n      (valueChange)=\"addSearchFilter('billingPositions.billingParties.linkedId', $event)\"\n    ></smallstack-rx-select>\n  </div>\n  <ng-container *ngIf=\"currentTodoTemplates$ | async as currentTodoTemplates\">\n    <table mat-table [dataSource]=\"currentTodoTemplates\" class=\"max-h-80 overflow-hidden scr\">\n      <ng-container matColumnDef=\"checkboxes\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\">\n          <input type=\"checkbox\" class=\"checkbox checkbox-primary\" (click)=\"toggleAll($event)\" [checked]=\"allChecked\" />\n        </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <input\n            type=\"checkbox\"\n            class=\"checkbox checkbox-primary\"\n            [checked]=\"isChecked(element.id)\"\n            (click)=\"toggle(element)\"\n          />\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"ownerIds\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\">Hausmeister</th>\n        <td mat-cell *matCellDef=\"let element\">\n          <smallstack-rx-property\n            *ngFor=\"let ownerId of element.ownerIds\"\n            type=\"customers\"\n            [modelId]=\"ownerId\"\n            property=\"displayName\"\n          >\n          </smallstack-rx-property>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"facility\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Geb\u00E4ude / Anlage </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <ng-container *ngFor=\"let billingPosition of element.billingPositions\">\n            <smallstack-rx-property\n              *ngFor=\"let billingParty of billingPosition.billingParties\"\n              [type]=\"billingParty.linkedType\"\n              [modelId]=\"billingParty.linkedId\"\n              property=\"label\"\n            >\n            </smallstack-rx-property\n          ></ng-container>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"label\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Aufgabe </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <smallstack-i18n [data]=\"element.label\"></smallstack-i18n>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"description\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Beschreibung </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <smallstack-i18n [data]=\"element.description\"></smallstack-i18n>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"date\">\n        <th mat-header-cell *matHeaderCellDef> Geplante Zeiten </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <smallstack-scheduled-times\n            [start]=\"(replacement$ | async)?.start\"\n            [end]=\"(replacement$ | async)?.end\"\n            [schedulingRule]=\"element.schedulingRule\"\n          ></smallstack-scheduled-times>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"actions\">\n        <th mat-header-cell *matHeaderCellDef> Vertreten durch </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <ng-container *ngIf=\"isAlreadyReplaced(element.id); else showSelect\"> Bereits vertreten </ng-container>\n          <ng-template #showSelect>\n            <select\n              class=\"select select-bordered select-primary select-sm\"\n              ngModel\n              (ngModelChange)=\"addReplacer($event, element.id)\"\n            >\n              <option></option>\n              <option *ngFor=\"let customer of customers$ | async\" [value]=\"customer.id\">\n                {{ customer.displayName }}\n              </option></select\n            >\n          </ng-template>\n        </td>\n      </ng-container>\n\n      <tr mat-header-row *matHeaderRowDef=\"newReplacersTableColumns\"></tr>\n      <tr mat-row *matRowDef=\"let row; columns: newReplacersTableColumns\"></tr>\n    </table>\n    <div *ngIf=\"checkedTodoTemplates && checkedTodoTemplates.length > 0\" class=\"flex flex-row gap-1 place-items-center\">\n      <div>Ausgew\u00E4hlte vertreten durch: </div>\n      <select class=\"select select-bordered select-primary select-sm\" [(ngModel)]=\"manyReplacerId\">\n        <option></option>\n        <option *ngFor=\"let customer of customers$ | async\" [value]=\"customer.id\">\n          {{ customer.displayName }}\n        </option></select\n      >\n      <button class=\"btn btn-primary btn-sm\" (click)=\"addManyReplacers()\">Hinzuf\u00FCgen</button>\n    </div>\n  </ng-container>\n  <div\n    *ngIf=\"\n      (currentTodoTemplates$ | async) === undefined ||\n      (currentTodoTemplates$ | async) === null ||\n      (currentTodoTemplates$ | async)?.length === 0\n    \"\n    class=\"text-center text-help m-6\"\n  >\n    Keine Aufgaben f\u00FCr die angegebenen Filter gefunden\n  </div>\n</div>\n", dependencies: [{ kind: "ngmodule", type: CommonModule }, { kind: "directive", type: i3.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "pipe", type: i3.AsyncPipe, name: "async" }, { kind: "ngmodule", type: FormsModule }, { kind: "directive", type: i4.NgSelectOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i4.ɵNgSelectMultipleOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i4.SelectControlValueAccessor, selector: "select:not([multiple])[formControlName],select:not([multiple])[formControl],select:not([multiple])[ngModel]", inputs: ["compareWith"] }, { kind: "directive", type: i4.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i4.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }, { kind: "ngmodule", type: MatLegacyTableModule }, { kind: "component", type: i5.MatLegacyTable, selector: "mat-table, table[mat-table]", exportAs: ["matTable"] }, { kind: "directive", type: i5.MatLegacyHeaderCellDef, selector: "[matHeaderCellDef]" }, { kind: "directive", type: i5.MatLegacyHeaderRowDef, selector: "[matHeaderRowDef]", inputs: ["matHeaderRowDef", "matHeaderRowDefSticky"] }, { kind: "directive", type: i5.MatLegacyColumnDef, selector: "[matColumnDef]", inputs: ["sticky", "matColumnDef"] }, { kind: "directive", type: i5.MatLegacyCellDef, selector: "[matCellDef]" }, { kind: "directive", type: i5.MatLegacyRowDef, selector: "[matRowDef]", inputs: ["matRowDefColumns", "matRowDefWhen"] }, { kind: "directive", type: i5.MatLegacyHeaderCell, selector: "mat-header-cell, th[mat-header-cell]" }, { kind: "directive", type: i5.MatLegacyCell, selector: "mat-cell, td[mat-cell]" }, { kind: "component", type: i5.MatLegacyHeaderRow, selector: "mat-header-row, tr[mat-header-row]", exportAs: ["matHeaderRow"] }, { kind: "component", type: i5.MatLegacyRow, selector: "mat-row, tr[mat-row]", exportAs: ["matRow"] }, { kind: "component", type: IconComponent, selector: "smallstack-icon", inputs: ["theme", "size", "name", "alias", "action", "actionSize", "spinning", "actionSpinning", "color", "themeColor", "actionColor", "badge", "dynamic", "badgeColor"], outputs: ["iconError"] }, { kind: "ngmodule", type: SmallstackFormCoreModule }, { kind: "component", type: i6.SchemaFormTableComponent, selector: "smallstack-form-table,schema-form-table,smallstack-schema-form-table", inputs: ["schema"] }, { kind: "component", type: i6.FormComponent, selector: "smallstack-form,[smallstack-form]", inputs: ["schema", "options", "value"], outputs: ["valueChange", "submitForm"] }, { kind: "component", type: I18nComponent, selector: "smallstack-i18n", inputs: ["data", "showMissingKey", "replacers", "locale", "translationStore"] }, { kind: "component", type: RxModelComponent, selector: "smallstack-rx-model", inputs: ["type", "modelId", "notFound"] }, { kind: "component", type: RxPropertyComponent, selector: "smallstack-rx-property", inputs: ["property", "notFound", "showNotFound"] }, { kind: "component", type: RxSelectComponent, selector: "smallstack-rx-select", inputs: ["type", "displayProperty", "valueProperty", "notFound", "label", "value", "showNotFound"], outputs: ["valueChange"] }, { kind: "component", type: ScheduledTimesComponent, selector: "smallstack-scheduled-times", inputs: ["start", "end", "schedulingRule"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.1.7", ngImport: i0, type: TodoTemplateOwnerReplacementEditorWidgetComponent, decorators: [{
            type: Component,
            args: [{ selector: "smallstack-todo-template-owner-replacement-widget", standalone: true, imports: [
                        CommonModule,
                        FormsModule,
                        MatLegacyTableModule,
                        MatSortStoreDirective,
                        StorePropertyComponent,
                        IconComponent,
                        SmallstackFormCoreModule,
                        DateComponent,
                        I18nComponent,
                        LoadingElementDirective,
                        RxModelComponent,
                        RxPropertyComponent,
                        RxSelectComponent,
                        ScheduledTimesComponent,
                        LetDirective
                    ], changeDetection: ChangeDetectionStrategy.OnPush, template: "<div class=\"p-2\">\n  <div class=\"text-lg font-bold\">Zeitraum</div>\n  <smallstack-form [schema]=\"formSchema\" (valueChange)=\"replacement$.next($event)\" [value]=\"replacement$ | async\">\n    <smallstack-form-table></smallstack-form-table>\n  </smallstack-form>\n  <div class=\"divider\"></div>\n  <div class=\"text-lg font-bold\">Aufgaben Vertretungen</div>\n  <table mat-table [dataSource]=\"(replacement$ | async)?.replacers\">\n    <ng-container matColumnDef=\"facility\">\n      <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Geb\u00E4ude / Anlage </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <smallstack-rx-model [type]=\"element.targetType\" [modelId]=\"element.targetId\">\n          <ng-template let-model=\"model\">\n            <ng-container *ngFor=\"let billingPosition of model?.billingPositions\">\n              <smallstack-rx-property\n                *ngFor=\"let billingParty of billingPosition.billingParties\"\n                [type]=\"billingParty.linkedType\"\n                [modelId]=\"billingParty.linkedId\"\n                property=\"label\"\n              ></smallstack-rx-property>\n            </ng-container>\n          </ng-template>\n        </smallstack-rx-model>\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"label\">\n      <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Aufgabe </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <smallstack-rx-model [type]=\"element.targetType\" [modelId]=\"element.targetId\">\n          <ng-template let-model=\"model\">\n            <smallstack-i18n [data]=\"model?.label\"></smallstack-i18n>\n          </ng-template>\n        </smallstack-rx-model>\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"description\">\n      <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Beschreibung </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <smallstack-rx-model [type]=\"element.targetType\" [modelId]=\"element.targetId\">\n          <ng-template let-model=\"model\">\n            <smallstack-i18n [data]=\"model?.description\"></smallstack-i18n>\n          </ng-template>\n        </smallstack-rx-model>\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"date\">\n      <th mat-header-cell *matHeaderCellDef> Geplante Zeiten </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <smallstack-rx-model [type]=\"element.targetType\" [modelId]=\"element.targetId\">\n          <ng-template let-model=\"model\">\n            <smallstack-scheduled-times\n              [start]=\"(replacement$ | async)?.start\"\n              [end]=\"(replacement$ | async)?.end\"\n              [schedulingRule]=\"model?.schedulingRule\"\n            ></smallstack-scheduled-times>\n          </ng-template>\n        </smallstack-rx-model>\n        <!-- <div class=\"flex flex-row flex-wrap gap-1\">\n          <div *ngFor=\"let date of element.dates\" class=\"badge badge-primary\">\n            <smallstack-date [timestamp]=\"date\"></smallstack-date>\n          </div>\n        </div> -->\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"replacers\">\n      <th mat-header-cell *matHeaderCellDef> Vertretung </th>\n      <td mat-cell *matCellDef=\"let element\" class=\"whitespace-nowrap\">\n        <ng-container *ngFor=\"let customer of element.newValue\">\n          <smallstack-rx-property type=\"customers\" [modelId]=\"customer\" property=\"displayName\">\n          </smallstack-rx-property>\n        </ng-container>\n      </td>\n    </ng-container>\n    <ng-container matColumnDef=\"actions\">\n      <th mat-header-cell *matHeaderCellDef> </th>\n      <td mat-cell *matCellDef=\"let element\">\n        <button class=\"btn-action btn-primary\" (click)=\"removeReplacer(element.targetId)\">\n          <smallstack-icon name=\"trash\"></smallstack-icon>\n        </button>\n      </td>\n    </ng-container>\n    <tr mat-header-row *matHeaderRowDef=\"replacersTableColumns\"></tr>\n    <tr mat-row *matRowDef=\"let row; columns: replacersTableColumns\"></tr>\n  </table>\n  <div\n    *ngIf=\"(replacement$ | async)?.replacers === undefined || (replacement$ | async)?.replacers?.length === 0\"\n    class=\"text-center text-help m-6\"\n  >\n    Keine Vertretungen vorhanden\n  </div>\n  <div class=\"divider\"></div>\n  <div class=\"text-lg font-bold\">Neue Vertretungen hinzuf\u00FCgen</div>\n  <div class=\"flex flex-row gap-2 py-3 outline-border\">\n    <div class=\"title\">Filter</div>\n    <!-- customer filter -->\n    <smallstack-rx-select\n      label=\"Hausmeister\"\n      type=\"customers\"\n      displayProperty=\"displayName\"\n      valueProperty=\"id\"\n      (valueChange)=\"addSearchFilter('ownerIds', $event)\"\n    ></smallstack-rx-select>\n\n    <!-- facility filter -->\n    <smallstack-rx-select\n      label=\"Geb\u00E4ude / Anlage\"\n      type=\"custom_facilities\"\n      displayProperty=\"label\"\n      valueProperty=\"id\"\n      (valueChange)=\"addSearchFilter('billingPositions.billingParties.linkedId', $event)\"\n    ></smallstack-rx-select>\n  </div>\n  <ng-container *ngIf=\"currentTodoTemplates$ | async as currentTodoTemplates\">\n    <table mat-table [dataSource]=\"currentTodoTemplates\" class=\"max-h-80 overflow-hidden scr\">\n      <ng-container matColumnDef=\"checkboxes\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\">\n          <input type=\"checkbox\" class=\"checkbox checkbox-primary\" (click)=\"toggleAll($event)\" [checked]=\"allChecked\" />\n        </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <input\n            type=\"checkbox\"\n            class=\"checkbox checkbox-primary\"\n            [checked]=\"isChecked(element.id)\"\n            (click)=\"toggle(element)\"\n          />\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"ownerIds\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\">Hausmeister</th>\n        <td mat-cell *matCellDef=\"let element\">\n          <smallstack-rx-property\n            *ngFor=\"let ownerId of element.ownerIds\"\n            type=\"customers\"\n            [modelId]=\"ownerId\"\n            property=\"displayName\"\n          >\n          </smallstack-rx-property>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"facility\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Geb\u00E4ude / Anlage </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <ng-container *ngFor=\"let billingPosition of element.billingPositions\">\n            <smallstack-rx-property\n              *ngFor=\"let billingParty of billingPosition.billingParties\"\n              [type]=\"billingParty.linkedType\"\n              [modelId]=\"billingParty.linkedId\"\n              property=\"label\"\n            >\n            </smallstack-rx-property\n          ></ng-container>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"label\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Aufgabe </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <smallstack-i18n [data]=\"element.label\"></smallstack-i18n>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"description\">\n        <th mat-header-cell *matHeaderCellDef class=\"whitespace-nowrap\"> Beschreibung </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <smallstack-i18n [data]=\"element.description\"></smallstack-i18n>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"date\">\n        <th mat-header-cell *matHeaderCellDef> Geplante Zeiten </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <smallstack-scheduled-times\n            [start]=\"(replacement$ | async)?.start\"\n            [end]=\"(replacement$ | async)?.end\"\n            [schedulingRule]=\"element.schedulingRule\"\n          ></smallstack-scheduled-times>\n        </td>\n      </ng-container>\n      <ng-container matColumnDef=\"actions\">\n        <th mat-header-cell *matHeaderCellDef> Vertreten durch </th>\n        <td mat-cell *matCellDef=\"let element\">\n          <ng-container *ngIf=\"isAlreadyReplaced(element.id); else showSelect\"> Bereits vertreten </ng-container>\n          <ng-template #showSelect>\n            <select\n              class=\"select select-bordered select-primary select-sm\"\n              ngModel\n              (ngModelChange)=\"addReplacer($event, element.id)\"\n            >\n              <option></option>\n              <option *ngFor=\"let customer of customers$ | async\" [value]=\"customer.id\">\n                {{ customer.displayName }}\n              </option></select\n            >\n          </ng-template>\n        </td>\n      </ng-container>\n\n      <tr mat-header-row *matHeaderRowDef=\"newReplacersTableColumns\"></tr>\n      <tr mat-row *matRowDef=\"let row; columns: newReplacersTableColumns\"></tr>\n    </table>\n    <div *ngIf=\"checkedTodoTemplates && checkedTodoTemplates.length > 0\" class=\"flex flex-row gap-1 place-items-center\">\n      <div>Ausgew\u00E4hlte vertreten durch: </div>\n      <select class=\"select select-bordered select-primary select-sm\" [(ngModel)]=\"manyReplacerId\">\n        <option></option>\n        <option *ngFor=\"let customer of customers$ | async\" [value]=\"customer.id\">\n          {{ customer.displayName }}\n        </option></select\n      >\n      <button class=\"btn btn-primary btn-sm\" (click)=\"addManyReplacers()\">Hinzuf\u00FCgen</button>\n    </div>\n  </ng-container>\n  <div\n    *ngIf=\"\n      (currentTodoTemplates$ | async) === undefined ||\n      (currentTodoTemplates$ | async) === null ||\n      (currentTodoTemplates$ | async)?.length === 0\n    \"\n    class=\"text-center text-help m-6\"\n  >\n    Keine Aufgaben f\u00FCr die angegebenen Filter gefunden\n  </div>\n</div>\n" }]
        }], ctorParameters: function () { return [{ type: i1.RxCollectionService }, { type: i2.AxiosApiClient }]; }, propDecorators: { formComponent: [{
                type: ViewChild,
                args: [FormComponent]
            }] } });

export { TodoTemplateOwnerReplacementEditorWidgetComponent };
//# sourceMappingURL=smallstack-todo-widgets-todo-template-owner-replacement-editor-widget.component-14942710.mjs.map
