import * as i3$1 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { Component, ChangeDetectionStrategy, Inject } from '@angular/core';
import { IconCardComponent } from '@smallstack/cms-components';
import { TRANSLATION_HELPER, LoadingElementDirective } from '@smallstack/common-components';
import { WIDGET_FORM_INPUT_I18N } from '@smallstack/core-common';
import * as i1$1 from '@smallstack/i18n-components';
import { DialogPopupButtons, DurationComponent, DateComponent, HumanizedDateRangeComponent } from '@smallstack/i18n-components';
import { StoreState } from '@smallstack/store';
import { LoaderComponent, StoreContainerComponent } from '@smallstack/store-components';
import { IconComponent } from '@smallstack/theme-components';
import * as i1 from '@smallstack/todo-components';
import * as i3 from '@smallstack/todo-shared';
import * as i1$2 from '@smallstack/typesystem-client';
import { TypeModelComponent } from '@smallstack/typesystem-client';
import { TYPE_MY_WORKLOGS } from '@smallstack/typesystem';
import { filterNullish, cloneObject } from '@smallstack/utils';
import { BaseWidgetComponent } from '@smallstack/widget-core';
import { map, switchMap } from 'rxjs';

class WorklogWidgetComponent extends BaseWidgetComponent {
    constructor(myWorklogStore, notificationService, translationHelper, todoConfigurationService, typeRegistry, todoStore) {
        super();
        this.myWorklogStore = myWorklogStore;
        this.notificationService = notificationService;
        this.translationHelper = translationHelper;
        this.typeRegistry = typeRegistry;
        this.todoStore = todoStore;
        this.worklogs$ = this.myWorklogStore.value$.pipe(filterNullish(), map((worklogs) => worklogs.sort((a, b) => b.startTime - a.startTime)), switchMap(async (worklogs) => {
            const expandedWorklogs = [];
            for (const worklog of worklogs) {
                const todoRef = await this.todoStore.get(worklog.todoRefId);
                expandedWorklogs.push({
                    ...worklog,
                    todoRef
                });
            }
            return expandedWorklogs;
        }));
        this.isLoading = true;
        if (myWorklogStore.state !== StoreState.LOADED)
            void myWorklogStore.preload();
        void todoConfigurationService.getConfiguration().then((config) => {
            this.todoConfiguration = config;
            this.isLoading = false;
            this.cdr.markForCheck();
        });
    }
    deleteWorklog(worklog) {
        return async () => {
            const label = this.translationHelper.translate(worklog.label);
            const confirmed = await this.notificationService.popup.confirmation("Protokoll löschen", `Möchten Sie das Arbeitsprotokoll "${label}" wirklich löschen? Dieser Vorgang kann nicht rückgängig gemacht werden.`, DialogPopupButtons.cancelDelete);
            if (confirmed === true)
                await this.myWorklogStore.delete(worklog.id);
        };
    }
    editWorklog(worklog) {
        return async () => {
            const worklogCopy = cloneObject(worklog);
            delete worklogCopy.todoRef;
            await this.typeRegistry.openEditor(TYPE_MY_WORKLOGS, worklogCopy, {
                schema: {
                    type: "object",
                    properties: {
                        label: {
                            title: "Bezeichnung",
                            "x-schema-form": { inputWidget: WIDGET_FORM_INPUT_I18N },
                            oneOf: [{ type: "string" }, { type: "array", items: { $ref: "#/components/schemas/InnerTranslation" } }]
                        },
                        startTime: { type: "number", title: "Start", "x-schema-form": { widget: "datetime" } },
                        endTime: { type: "number", title: "Ende", "x-schema-form": { widget: "datetime" } }
                    }
                }
            });
            await this.myWorklogStore.load();
        };
    }
    getNormalizedDuration(duration) {
        if (this.todoConfiguration.billingRoundingMinutes)
            return this.getRoundedDuration(this.todoConfiguration, duration);
        else
            return duration;
    }
    // TODO: copied from api/invoice service
    getRoundedDuration(todoConfiguration, duration) {
        if (typeof todoConfiguration?.billingRoundingMinutes === "number" &&
            todoConfiguration.billingRoundingMinutes > 0 &&
            todoConfiguration.billingRoundingMinutes <= 60) {
            const billingRoundingMinutesInMs = todoConfiguration.billingRoundingMinutes * 60 * 1000;
            const durationTimes = Math.ceil(duration / billingRoundingMinutesInMs);
            duration = durationTimes * billingRoundingMinutesInMs;
        }
        return duration;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.1.7", ngImport: i0, type: WorklogWidgetComponent, deps: [{ token: i1.MyWorklogStore }, { token: i1$1.NotificationService }, { token: TRANSLATION_HELPER }, { token: i3.TodoConfigurationService }, { token: i1$2.FrontendTypeRegistry }, { token: i1.TodoStore }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.1.7", type: WorklogWidgetComponent, isStandalone: true, selector: "ng-component", usesInheritance: true, ngImport: i0, template: "<smallstack-loader *ngIf=\"isLoading; else content\" [size]=\"'small'\"></smallstack-loader>\n\n<ng-template #content>\n  <smallstack-store-container [store]=\"myWorklogStore\">\n    <ng-template #empty>\n      <div class=\"list-container-empty\">\n        <smallstack-icon name=\"time\"></smallstack-icon>\n        <div>Keine Arbeitsprotokolle vorhanden.</div></div\n      ></ng-template\n    >\n    <div class=\"flex flex-col\">\n      <smallstack-icon-card\n        *ngFor=\"let worklog of worklogs$ | async; trackByIndex\"\n        icon=\"time\"\n        [title]=\"worklog.label\"\n        [subTitle]=\"worklog.todoRef?.description\"\n      >\n        <div class=\"flex flex-col space-y-2\">\n          <div class=\"date text-muted flex flex-row space-x-1 items-center\">\n            <smallstack-icon name=\"company\" size=\"24\"></smallstack-icon>\n            <ng-container *ngFor=\"let billingPosition of worklog.todoRef?.billingPositions\">\n              <ng-container *ngFor=\"let billingParty of billingPosition.billingParties\">\n                <smallstack-type-model [modelId]=\"billingParty.linkedId\" [typePath]=\"billingParty.linkedType\">\n                  <ng-template let-short=\"short\" let-long=\"long\">\n                    <b>{{ short }}</b>\n                    <div class=\"text-muted\">{{ long }}</div>\n                  </ng-template>\n                </smallstack-type-model>\n              </ng-container>\n            </ng-container>\n          </div>\n          <div class=\"date text-muted flex flex-row space-x-1 items-center\">\n            <smallstack-icon name=\"time-machine--v1\" size=\"24\"></smallstack-icon>\n            <div class=\"flex flex-col\">\n              <div *ngIf=\"worklog.todoRef?.dueDate\" class=\"flex flex-row space-x-1\">\n                <b>Geplante Beauftragung</b>\n                <smallstack-date [timestamp]=\"worklog.todoRef.dueDate\"></smallstack-date>\n              </div>\n              <div class=\"flex flex-row space-x-1\">\n                <b>Ausf\u00FChrung</b>\n                <smallstack-humanized-date-range\n                  [start]=\"worklog.startTime\"\n                  [end]=\"worklog.endTime\"\n                ></smallstack-humanized-date-range>\n              </div>\n            </div>\n          </div>\n          <div *ngIf=\"worklog.endTime\" class=\"date text-muted flex flex-row space-x-1 items-center\">\n            <smallstack-icon name=\"stopwatch\" size=\"24\"></smallstack-icon>\n            <smallstack-duration [duration]=\"getNormalizedDuration(worklog.duration)\"></smallstack-duration>\n          </div>\n          <div class=\"flex flex-row gap-2 justify-evenly\">\n            <button *ngIf=\"data.showEditBtn\" [loadingFn]=\"editWorklog(worklog)\" class=\"btn btn-primary flex-auto\">\n              Bearbeiten\n            </button>\n            <button *ngIf=\"data.showDeleteBtn\" [loadingFn]=\"deleteWorklog(worklog)\" class=\"btn btn-error flex-auto\">\n              L\u00F6schen\n            </button>\n          </div>\n        </div>\n      </smallstack-icon-card>\n    </div>\n  </smallstack-store-container>\n</ng-template>\n", styles: [".worklog-entry{background-color:#fff;border-radius:8px;margin:16px 0;padding:16px;box-shadow:0 2px 1px -1px #0003,0 1px 1px #00000024,0 1px 3px #0000001f}.worklog-entry .title{max-width:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;margin-bottom:12px}.worklog-entry .date{font-size:13px;padding-left:10px}.worklog-entry .smallstack-icon{margin-right:4px}.list-container-empty{-webkit-user-select:none;user-select:none;align-items:center;text-align:center;padding:50px;color:#898989;font-weight:100;font-size:16px;font-feature-settings:\"pcap\",\"c2pc\";font-variant:all-petite-caps}\n"], dependencies: [{ kind: "ngmodule", type: CommonModule }, { kind: "directive", type: i3$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i3$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "pipe", type: i3$1.AsyncPipe, name: "async" }, { kind: "component", type: LoaderComponent, selector: "smallstack-loader", inputs: ["loading", "color", "size", "loadingText"] }, { kind: "component", type: StoreContainerComponent, selector: "smallstack-store-container", inputs: ["store", "loadingText", "showErrorIcon", "showErrorText"] }, { kind: "component", type: IconComponent, selector: "smallstack-icon", inputs: ["theme", "size", "name", "alias", "action", "actionSize", "spinning", "actionSpinning", "color", "themeColor", "actionColor", "badge", "dynamic", "badgeColor"], outputs: ["iconError"] }, { kind: "component", type: IconCardComponent, selector: "smallstack-icon-card", inputs: ["icon", "title", "subTitle", "height", "link"] }, { kind: "component", type: TypeModelComponent, selector: "smallstack-type-model", inputs: ["modelId", "typePath", "displayShort", "displayLong"] }, { kind: "directive", type: LoadingElementDirective, selector: "[loadingFn]", inputs: ["loadingFn", "isLoading"], exportAs: ["loadingFn"] }, { kind: "component", type: DurationComponent, selector: "smallstack-duration", inputs: ["start", "end", "duration"] }, { kind: "component", type: DateComponent, selector: "smallstack-date", inputs: ["timestamp", "format"] }, { kind: "component", type: HumanizedDateRangeComponent, selector: "smallstack-humanized-date-range", inputs: ["start", "end"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.1.7", ngImport: i0, type: WorklogWidgetComponent, decorators: [{
            type: Component,
            args: [{ changeDetection: ChangeDetectionStrategy.OnPush, standalone: true, imports: [
                        CommonModule,
                        LoaderComponent,
                        StoreContainerComponent,
                        IconComponent,
                        IconCardComponent,
                        TypeModelComponent,
                        LoadingElementDirective,
                        DurationComponent,
                        DateComponent,
                        HumanizedDateRangeComponent
                    ], template: "<smallstack-loader *ngIf=\"isLoading; else content\" [size]=\"'small'\"></smallstack-loader>\n\n<ng-template #content>\n  <smallstack-store-container [store]=\"myWorklogStore\">\n    <ng-template #empty>\n      <div class=\"list-container-empty\">\n        <smallstack-icon name=\"time\"></smallstack-icon>\n        <div>Keine Arbeitsprotokolle vorhanden.</div></div\n      ></ng-template\n    >\n    <div class=\"flex flex-col\">\n      <smallstack-icon-card\n        *ngFor=\"let worklog of worklogs$ | async; trackByIndex\"\n        icon=\"time\"\n        [title]=\"worklog.label\"\n        [subTitle]=\"worklog.todoRef?.description\"\n      >\n        <div class=\"flex flex-col space-y-2\">\n          <div class=\"date text-muted flex flex-row space-x-1 items-center\">\n            <smallstack-icon name=\"company\" size=\"24\"></smallstack-icon>\n            <ng-container *ngFor=\"let billingPosition of worklog.todoRef?.billingPositions\">\n              <ng-container *ngFor=\"let billingParty of billingPosition.billingParties\">\n                <smallstack-type-model [modelId]=\"billingParty.linkedId\" [typePath]=\"billingParty.linkedType\">\n                  <ng-template let-short=\"short\" let-long=\"long\">\n                    <b>{{ short }}</b>\n                    <div class=\"text-muted\">{{ long }}</div>\n                  </ng-template>\n                </smallstack-type-model>\n              </ng-container>\n            </ng-container>\n          </div>\n          <div class=\"date text-muted flex flex-row space-x-1 items-center\">\n            <smallstack-icon name=\"time-machine--v1\" size=\"24\"></smallstack-icon>\n            <div class=\"flex flex-col\">\n              <div *ngIf=\"worklog.todoRef?.dueDate\" class=\"flex flex-row space-x-1\">\n                <b>Geplante Beauftragung</b>\n                <smallstack-date [timestamp]=\"worklog.todoRef.dueDate\"></smallstack-date>\n              </div>\n              <div class=\"flex flex-row space-x-1\">\n                <b>Ausf\u00FChrung</b>\n                <smallstack-humanized-date-range\n                  [start]=\"worklog.startTime\"\n                  [end]=\"worklog.endTime\"\n                ></smallstack-humanized-date-range>\n              </div>\n            </div>\n          </div>\n          <div *ngIf=\"worklog.endTime\" class=\"date text-muted flex flex-row space-x-1 items-center\">\n            <smallstack-icon name=\"stopwatch\" size=\"24\"></smallstack-icon>\n            <smallstack-duration [duration]=\"getNormalizedDuration(worklog.duration)\"></smallstack-duration>\n          </div>\n          <div class=\"flex flex-row gap-2 justify-evenly\">\n            <button *ngIf=\"data.showEditBtn\" [loadingFn]=\"editWorklog(worklog)\" class=\"btn btn-primary flex-auto\">\n              Bearbeiten\n            </button>\n            <button *ngIf=\"data.showDeleteBtn\" [loadingFn]=\"deleteWorklog(worklog)\" class=\"btn btn-error flex-auto\">\n              L\u00F6schen\n            </button>\n          </div>\n        </div>\n      </smallstack-icon-card>\n    </div>\n  </smallstack-store-container>\n</ng-template>\n", styles: [".worklog-entry{background-color:#fff;border-radius:8px;margin:16px 0;padding:16px;box-shadow:0 2px 1px -1px #0003,0 1px 1px #00000024,0 1px 3px #0000001f}.worklog-entry .title{max-width:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;margin-bottom:12px}.worklog-entry .date{font-size:13px;padding-left:10px}.worklog-entry .smallstack-icon{margin-right:4px}.list-container-empty{-webkit-user-select:none;user-select:none;align-items:center;text-align:center;padding:50px;color:#898989;font-weight:100;font-size:16px;font-feature-settings:\"pcap\",\"c2pc\";font-variant:all-petite-caps}\n"] }]
        }], ctorParameters: function () { return [{ type: i1.MyWorklogStore }, { type: i1$1.NotificationService }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [TRANSLATION_HELPER]
                }] }, { type: i3.TodoConfigurationService }, { type: i1$2.FrontendTypeRegistry }, { type: i1.TodoStore }]; } });

export { WorklogWidgetComponent };
//# sourceMappingURL=smallstack-todo-widgets-worklog-widget.component-0db7b867.mjs.map
